/*
 *
 *
 */

#include <defs.h>


int aim_parse_incomingauth(struct aim_session_t *sess, struct command_rx_struct *cur)
{
  int i;
  int snlen;
  char *snbuf = NULL;
  struct aimd_regusers *user;
  struct aimd_clientinfo *client;
  struct aim_tlvlist_t *tlvlist = NULL;

  i = 6; /* first few bytes already verified by dispatcher */

  snlen = aimutil_get16(cur->data+i);
  i += 2;
  
  /*
   * Check validity of screen name formatting
   */
  if ((snlen < 5) || (snlen > 16)) {
    sess->logininfo.errorcode = 0x0001; /* invalid screen name */
    sess->logininfo.errorurl = malloc(strlen(LOGINERR_URL_INVALSN)+1);
    strcpy(sess->logininfo.errorurl, LOGINERR_URL_INVALSN);
    
    /* 
     * An invalid SN error must be handled with care.  We cannot
     * use the sn buffer in the session because its limited in size.
     * The error needs to return the full SN that was requested.
     */
    snbuf = malloc(snlen+1);
    memcpy(snbuf, cur->data+i, snlen);
    snbuf[snlen] = '\0';
    
    TXCALLLOCKED(aim_sendauthresp(sess, cur->conn, snbuf, NULL, NULL, NULL, 0);)
    free(snbuf);
    free(sess->logininfo.errorurl);

    return 1;
  }

  /*
   * Read screen name
   */
  memcpy(sess->logininfo.screen_name, cur->data+i, snlen);
  sess->logininfo.screen_name[snlen] = '\0';

  
  /*
   * Make sure its a valid user. 
   */
  if (!(user = aimd_findreguser(sess->logininfo.screen_name))) {
    sess->logininfo.errorcode = 0x0001; /* invalid screen name */
    sess->logininfo.errorurl = malloc(strlen(LOGINERR_URL_INVALSN)+1);
    strcpy(sess->logininfo.errorurl, LOGINERR_URL_INVALSN);

    TXCALLLOCKED(aim_sendauthresp(sess, cur->conn, NULL, NULL, NULL, NULL, 0);)
    free(sess->logininfo.errorurl);

    return 1;
  }

  /*
   * We can continue parsing using only TLVs
   */
  tlvlist = aim_readtlvchain(cur->data+i, cur->commandlen-i);

  /*
   * We know its a valid user.  Check password.
   */
  if (aim_gettlv(tlvlist, 0x0002, 1)) {
    char *password;
    char *decoded = NULL;
    int z;
    int failure = 0;

    password = aim_gettlv_str(tlvlist, 0x0002, 1);
    decoded = (char *)malloc(strlen(password)+1);

    /*
     * Because AIM uses a completly reversible password
     * "encryption", we can use the encode function for 
     * both directions.
     */
    aim_encode_password(password, decoded);
    decoded[strlen(password)] = '\0';
    
    for (z = 0; z < strlen(password); z++) {
      if (decoded[z] != user->password[z]) {
	failure = 1;
	break;
      }
    }

    if (failure) {
      sess->logininfo.errorcode = 0x0005; /* invalid password */
      sess->logininfo.errorurl = malloc(strlen(LOGINERR_URL_INVALPW)+1);
      strcpy(sess->logininfo.errorurl, LOGINERR_URL_INVALPW);
    
      TXCALLLOCKED(aim_sendauthresp(sess, cur->conn, NULL, NULL, NULL, NULL, 0);)
      free(sess->logininfo.errorurl);

      return 1;
    } 
    
  }

  /*
   * Authorization was successful.  
   *
   * We could continue here and store various bits of client inforamtion.
   * But we have little reason to keep track of that.
   *
   * Go ahead and store the struct and get ready to redirect to BOS.
   *
   */
  sess->logininfo.errorcode = 0;

  client = aimd_addclient(sess->logininfo.screen_name, sess);
  client->user = user;
  aim_gencookie(client->lastcookie);
  client->lastservrequested = AIM_CONN_TYPE_BOS; /* BOS */
  strcpy(client->assignedbos, BOSIP);

  TXCALLLOCKED(aim_sendauthresp(sess, cur->conn, NULL, client->assignedbos, client->lastcookie, user->email, user->regstatus););
  
  /* XXX: eewwww */
  {	
    struct aimd_clientinfo *curcl;
    pthread_mutex_lock(&clientlist_lock);
    for (curcl = clientlist; curcl; curcl = curcl->next) {
      TXCALLLOCKED(aim_sendbuddyoncoming(curcl->sess, aim_getconn_type(curcl->sess, AIM_CONN_TYPE_BOS), &user->userinfo););
    }
    pthread_mutex_unlock(&clientlist_lock);
  }

  
  return 1;
}
