 /* mfaim
 * Copyright (c) 1999 Josh Myer <josh@joshisanerd.com>
 * console-mode AIM client
 *
 * mfaim is based on naim;
 *  naim is Copyright (c) 1998, 1999 Daniel Reed <n@ml.org>
 * 
 * mfaim also uess libfaim,
 *  FAIM is Copyright 1998 Adam Fritzler <afritz@iname.com>
 *  FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
 */

#include <mfaim.h>

extern WINDOW	*win_buddy, *win_status, *win_display;
extern faimconf_t	faimconf;
extern mfaimses_t       mfaimses;
extern struct aim_session_t mfaim_session;
extern int input_over;

buddywin_t	nullbwin, *curbwin;

void bsetup(void) {
  curbwin = &nullbwin;
  bzero(curbwin, sizeof(buddywin_t));
  strncpy(curbwin->winname, "=Status=", MAXSNLEN);
  curbwin->win = win_status;
  curbwin->next = curbwin;
  curbwin->prev = curbwin;
  wbuddy_update();
}

void wbuddy_update(void) {
  buddywin_t	*bwin = curbwin;
  time_t  t;

  wundirty(bwin);
  t = time(NULL);

  werase(win_buddy);

  do {
    if (bwin->waiting) {
      wprintf(win_buddy, BUDDY_WAITING, 1,"(%s)", bwin->winname);
    } else if (bwin->offline) {
      wprintf(win_buddy, BUDDY_OFFLINE, 0,"<%s>", bwin->winname);
    } else if (bwin->notbuddy) {
      wprintf(win_buddy, BUDDY_NONBUDDY, 1,"{%s}", bwin->winname);
    } else if (bwin->away) {
      wprintf(win_buddy, BUDDY_TEXT, 1, "_%s_", bwin->winname);
    } else if (bwin->chat) {
      wprintf(win_buddy, BUDDY_NONBUDDY, 1, "^%s^", bwin->winname);
    } else {
      wprintf(win_buddy, BUDDY_TEXT, 1,"[%s]", bwin->winname);
    }
  } while ((bwin = bwin->next) != curbwin);
  
  t = (t - mfaimses.online)/60;
  wdisplay_update(t);
  return;
}

void wdirty(buddywin_t *BW) { /* just in case we want to Do More things */
  BW->waiting = 1;	 /* in the future ;^)*/
  BW->keepafterso = 1;
#if USE_STACK
  sn_queue_enq(BW->winname);
#endif
  return;
}

void wundirty(buddywin_t *BW) {
  BW->waiting = 0;
#if USE_STACK
  sn_queue_rm(BW->winname);
#endif
  return;
}

void wdisplay_update(time_t t) {
  struct tm       *tmptr;
  int pos = 0;
  time_t now;

  char format[10]; /* arbitrarily large; note bzero below */
  char *display_line;

  now = time(NULL);

  display_line = malloc((WDISPLAY_X*WDISPLAY_Y)+1);

  bzero(format, 10);
  bzero(display_line, ((WDISPLAY_X*WDISPLAY_Y)+1));

  tmptr = localtime(&now);

  t = ((now - mfaimses.online)/60);
  
  snprintf(display_line, (WDISPLAY_X * WDISPLAY_Y), "%02i:%02i [mfaim pre] %s %s : Online %4lu%s ", tmptr->tm_hour, tmptr->tm_min, mfaimses.sn, ((input_over==1)?"(Overwrite)":""),((t/60>1)?(t/60):(t&255)), ((t/60>1)?"h":"m") ); /* whee!!!!  */
  
  snprintf(format, 10, "%%-%ds", strlen(display_line));

  pos = strlen(display_line);

  werase(win_display);
  wprintf(win_display, BUDDY_INFO, 1, format, display_line);
  bzero(display_line, ((WDISPLAY_X * WDISPLAY_Y)+1) );

#if USE_STACK
  if(sn_queue_peek() != NULL) {
    snprintf(format, 10, "%%-%ds", (strlen(sn_queue_peek())+2));
    snprintf(display_line, (WDISPLAY_X * WDISPLAY_Y)-pos, "<%s>", sn_queue_peek());    
    pos += strlen(sn_queue_peek())+2;
    wprintf(win_display, BUDDY_WAITING, 1, format, display_line);
  }
#endif

  snprintf(format, 10, "%%-%ds",(WDISPLAY_X * WDISPLAY_Y)-pos);
  bzero(display_line, (WDISPLAY_X * WDISPLAY_Y));
  memset(display_line, ' ', (WDISPLAY_X * WDISPLAY_Y)-pos);
  wprintf(win_display, BUDDY_INFO, 1, format, display_line);
	  
  free(display_line);
  return;
}

int bonline(char *buddy) {
  buddywin_t *bwin = curbwin;

  do /* if this isn't pretty code, i don't know what is =) */
    if (aimcmp(buddy, bwin->winname) == 0)
      if(!(bwin->chat) && !(bwin->offline))
	return(1);
  while ((bwin = bwin->next) != curbwin);
  
  return(0);
}

buddywin_t *wgetwin(char *winname) {
  buddywin_t *bwin = curbwin;
  do
    if(aimcmp(winname, bwin->winname) == 0)
      return(bwin);
  while((bwin = bwin->next) != curbwin);
  return(&nullbwin);
}

buddywin_t *wgetwin_u(char *winname) { /* partial unique substr match */
  buddywin_t *bwin = curbwin;
  char snbuf[MAXWLEN+1];

  sn_normalize(winname, winname);
  sn_normalize(snbuf, bwin->winname);
  /* XXX: /j <substr curbwin> ?? */

  do{
    sn_normalize(snbuf, bwin->winname);
    if (strstr(snbuf,winname) == snbuf)
      return(bwin);
  } while((bwin = bwin->next) != curbwin);

  return(NULL);
}

buddywin_t *bgetwin(char *buddy) { /* exact aim match */
  buddywin_t	*bwin = curbwin;

  do
    if (aimcmp(buddy, bwin->winname) == 0)
      if(!(bwin->chat))
	return(bwin);
  while ((bwin = bwin->next) != curbwin);

  return(&nullbwin);
}

buddywin_t *bgetwin_u(char *buddy) { /* partial unique substr match */
  buddywin_t *bwin = curbwin;
  char snbuf[MAXSNLEN+1];

  sn_normalize(buddy, buddy);
  sn_normalize(snbuf, bwin->winname);
  /* XXX: /j <substr curbwin> ?? */

  do{
    sn_normalize(snbuf, bwin->winname);
    if (strstr(snbuf,buddy) == snbuf)
      if(!(bwin->chat))
	return(bwin);
  } while((bwin = bwin->next) != curbwin);

  return(NULL);
}

buddywin_t *cgetwin(char *chatroom) { /* exact aim match */
  buddywin_t *bwin = curbwin;

  do 
    if (aimcmp(chatroom, bwin->winname) == 0)
      if(bwin->chat)
	return(bwin);
  while ((bwin = bwin->next) != curbwin);

  return(&nullbwin);
}


buddywin_t *cgetwin_u(char *chatroom) { /* partial unique substr match */
  buddywin_t *bwin = curbwin;
  char snbuf[MAXCRLEN+1];

  cr_normalize(chatroom, chatroom);
  cr_normalize(snbuf, bwin->winname);

  while((bwin = bwin->next) != curbwin) {
    cr_normalize(snbuf, bwin->winname);
    if (strstr(snbuf,chatroom) == snbuf)
      if(bwin->chat)
	return(bwin);
  }
  return(NULL);
}

 
void wremove(buddywin_t *bwin) {
  if (bwin->chat)
    aim_chat_leaveroom(&mfaim_session, bwin->winname);
  if(bwin->win != nullbwin.win)
    delwin(bwin->win);
  bwin->win = nullbwin.win;
#if USE_STACK
  sn_queue_rm(bwin->winname);
#endif
  return;
}

#define NWIN(WIN) newwin(faimconf.WIN.widthy, faimconf.WIN.widthx, \
	faimconf.WIN.starty, faimconf.WIN.startx);

void	bnewwin(char *buddy) {
  buddywin_t *bwin = NULL;
  
  if((bwin = (buddywin_t *)calloc(1, sizeof(buddywin_t))) == NULL) {
    printf("Couldn't calloc for bwin %s!\n", buddy);
    exit(1);
  }

  bwin->keepafterso = bwin->notbuddy = bwin->waiting = 0;
  bwin->chat = 0;
  bwin->offline = 1;
#if USE_STACK
  sn_queue_rm(bwin->winname);
#endif
  bzero(bwin->winname, MAXSNLEN + 1);
  strncpy(bwin->winname, buddy, MAXSNLEN);

  bwin->win = bnewwindow();


  bwin->next = curbwin->next;
  bwin->prev = curbwin;
  bwin->next->prev = bwin;
  curbwin->next = bwin;

  return;
}

void	cnewwin(char *room) {
  buddywin_t *bwin = NULL;
  
  if((bwin = (buddywin_t *)calloc(1, sizeof(buddywin_t))) == NULL) {
    printf("Couldn't calloc for bwin %s!\n", room);
    exit(1);
  }

  bwin->keepafterso = bwin->notbuddy = bwin->waiting  = 0;
  bwin->chat = 1;
#if USE_STACK
  sn_queue_rm(bwin->winname);
#endif
  bzero(bwin->winname, MAXCRLEN + 1);
  strncpy(bwin->winname, room, MAXCRLEN);

  bwin->win = bnewwindow();

  bwin->offline = 1;
  bwin->chat = 0;

  bwin->next = curbwin->next;
  bwin->prev = curbwin;
  bwin->next->prev = bwin;
  curbwin->next = bwin;

  return;
}


WINDOW* bnewwindow(void) {	

  WINDOW *win = NWIN(wstatus);
  winitwin(win, IMWIN_BGCOLOR);
  return(win);

}


void bcoming(char *buddy) {
  buddywin_t *bwin = NULL;
  buddylist_t *blist = NULL;
  
  assert(buddy != NULL);

  if ((blist = rgetlist(&faimconf, buddy)) == NULL) {
    printf("Er, I just received a logon notice for %s, but I can't seem to find %s in my buddy list...\n", buddy, buddy);
    return;
  }
  if ((bwin = wgetwin(buddy)) == &nullbwin) {
    bnewwin(buddy);
    bwin = wgetwin(buddy);
    assert(bwin != NULL);
  }
  if (bwin->offline == 1) {
    bwin->offline = 0;
    logim("LOGIN",buddy," ");

    mfaim_dprintf(bwin->win, IMWIN_STATUS, "%s %s is now online =)<BR>", blist->group, buddy);
    if (secs_getvar_int("beeponsignon") == 1)
      beep();
  }
  bwin->notbuddy = 0;
  bwin->chat = 0;
  bwin->away = 0;
  wbuddy_update();
}

void bgoing(char *buddy) {
  buddywin_t *bwin = curbwin;
  buddylist_t *blist = NULL;
  
  logim("LOGOFF",buddy," ");
  
  if ((blist = rgetlist(&faimconf, buddy)) == NULL)
    printf("Er, I just received a logoff notice for %s, but I can't seem to find %s in my buddy list...\n", buddy, buddy);
  
  bwin = bgetwin(buddy);

  /* XXX: decide whether auto-close is really worthwhile.  this code
     has issues with deciding when to close, not quite sensical, but
     worthwhile if you're into this sort of thing.
  */

#if 0
  if(bwin->keepafterso != 0 && bwin != &nullbwin) {
    buddywin_t *bbefore = bwin->next,
      *bnext = bwin->next;	
    while (bbefore->next != bwin)
      bbefore = bbefore->next;
    if (curbwin == bwin)
      curbwin = bnext;
    wremove(bwin);
    bbefore->next = bnext;
    bnext->prev = bbefore;
    wbuddy_update();
    touchwin(curbwin->win);
    bwin = bgetwin(buddy); /* XXX: buddy and chat room with same name. */
  }
#endif
  mfaim_dprintf(bwin->win, IMWIN_STATUS, "%s %s has just logged off<BR>", blist->group, buddy);
  //wdirty(bwin);
  if(bwin != &nullbwin)
    bwin->offline = 1;

  wbuddy_update();

  if (secs_getvar_int("beeponsignon") == 1)
    beep();

  return;
}

void bclearall(void) {
  buddywin_t	*bwin, *bnext = nullbwin.next;
  buddylist_t	*blist = NULL;

  assert(bnext != NULL);
  debug_printf("\tbclearall(), curbwin = %p\n", curbwin);
  curbwin = &nullbwin;
  debug_printf("\tbclearall(), curbwin = %p\n", curbwin);
  while ((bwin = bnext) != &nullbwin) {
    if( (blist = rgetlist(&faimconf, bwin->winname)) ) {
      bnext = bwin->next;
      if (bwin->keepafterso == 1) {
	bwin->offline = 1;
	wprintf(bwin->win, IMWIN_STATUS, 1,
		"%s %s has just"
		" logged off :(<BR>",
		blist->group, 
		bwin->winname);
      } else {
	buddywin_t	*bbefore = bwin->next;
	
	while (bbefore->next != bwin)
	  bbefore = bbefore->next;
	wremove(bwin);
	bbefore->next = bnext;
      }
    } else {
      wprintf(bwin->win, IMWIN_STATUS, 1, 
	      "%s has just logged off. They're not on your buddylist though.<BR>", 
	      bwin->winname);
    }
      wbuddy_update();
  }
  debug_printf("\tbclearall(), curbwin = %p\n", curbwin);
}
